@@ -22,6 +22,16 @@ IRONIC_INSPECTOR_STANDALONE=${IRONIC_INSPECTOR_STANDALONE:-True}
 IRONIC_INSPECTOR_UWSGI=$IRONIC_INSPECTOR_BIN_DIR/ironic-inspector-api-wsgi
 IRONIC_INSPECTOR_UWSGI_CONF=$IRONIC_INSPECTOR_CONF_DIR/ironic-inspector-uwsgi.ini
 
+# Determine if ironic is in enforce scope node, infer that to mean our operating mode
+# explicitly unless otherwise set.
+IRONIC_INSPECTOR_ENFORCE_SCOPE=${IRONIC_INSPECTOR_ENFORCE_SCOPE:-${IRONIC_ENFORCE_SCOPE:-False}}
+# and then fallback to trueorfalse to put it into the standardized string format for the jobs.
+IRONIC_INSPECTOR_ENFORCE_SCOPE=$(trueorfalse False IRONIC_INSPECTOR_ENFORCE_SCOPE)
+# Reset the input in the event the plugin is running separately from ironic's
+# devstack plugin.
+IRONIC_ENFORCE_SCOPE=$(trueorfalse False IRONIC_ENFORCE_SCOPE)
+
+
 if [[ -n ${IRONIC_INSPECTOR_MANAGE_FIREWALL} ]] ; then
     echo "IRONIC_INSPECTOR_MANAGE_FIREWALL is deprecated." >&2
     echo "Please, use IRONIC_INSPECTOR_DHCP_FILTER == noop/iptables/dnsmasq instead." >&2
@@ -126,7 +136,8 @@ function inspector_iniset {
 
 function install_inspector {
     setup_develop $IRONIC_INSPECTOR_DIR
-
+    # Check if things look okay
+    ironic-inspector-status upgrade check
     if [[ "$IRONIC_INSPECTOR_STANDALONE" == "False" ]]; then
         install_apache_wsgi
         # NOTE(rpittau) since devstack doesn't install test-requirements
@@ -197,7 +208,8 @@ function prepare_tftp {
     IRONIC_INSPECTOR_INITRAMFS_PATH="$IRONIC_INSPECTOR_IMAGE_PATH.initramfs"
     IRONIC_INSPECTOR_CALLBACK_URI="$IRONIC_INSPECTOR_INTERNAL_URI/v1/continue"
 
-    IRONIC_INSPECTOR_KERNEL_CMDLINE="$IRONIC_INSPECTOR_EXTRA_KERNEL_CMDLINE ipa-inspection-callback-url=$IRONIC_INSPECTOR_CALLBACK_URI"
+    IRONIC_INSPECTOR_KERNEL_CMDLINE="root=/dev/ram0 $IRONIC_INSPECTOR_EXTRA_KERNEL_CMDLINE"
+    IRONIC_INSPECTOR_KERNEL_CMDLINE="$IRONIC_INSPECTOR_KERNEL_CMDLINE ipa-inspection-callback-url=$IRONIC_INSPECTOR_CALLBACK_URI"
     IRONIC_INSPECTOR_KERNEL_CMDLINE="$IRONIC_INSPECTOR_KERNEL_CMDLINE ipa-api-url=$SERVICE_PROTOCOL://$SERVICE_HOST/baremetal"
     IRONIC_INSPECTOR_KERNEL_CMDLINE="$IRONIC_INSPECTOR_KERNEL_CMDLINE ipa-insecure=1 systemd.journald.forward_to_console=yes"
     IRONIC_INSPECTOR_KERNEL_CMDLINE="$IRONIC_INSPECTOR_KERNEL_CMDLINE vga=normal console=tty0 console=ttyS0"
@@ -229,7 +241,7 @@ function prepare_tftp {
 
 dhcp
 
-kernel http://$IRONIC_HTTP_SERVER:$IRONIC_HTTP_PORT/ironic-inspector.kernel BOOTIF=\${mac} $IRONIC_INSPECTOR_KERNEL_CMDLINE
+kernel http://$IRONIC_HTTP_SERVER:$IRONIC_HTTP_PORT/ironic-inspector.kernel BOOTIF=\${mac} $IRONIC_INSPECTOR_KERNEL_CMDLINE initrd=ironic-inspector.initramfs
 initrd http://$IRONIC_HTTP_SERVER:$IRONIC_HTTP_PORT/ironic-inspector.initramfs
 boot
 EOF
@@ -253,11 +265,20 @@ EOF
 function inspector_configure_auth_for {
     inspector_iniset $1 auth_type password
     inspector_iniset $1 auth_url "$KEYSTONE_SERVICE_URI"
-    inspector_iniset $1 username $IRONIC_INSPECTOR_ADMIN_USER
-    inspector_iniset $1 password $SERVICE_PASSWORD
-    inspector_iniset $1 project_name $SERVICE_PROJECT_NAME
+    if [[ "$1" == "ironic" ]] && [[ "$IRONIC_ENFORCE_SCOPE" == "True" ]]; then
+        # If ironic is enforcing scope, service credentials are not
+        # enough, because they live in a "service project" and does not
+        # have a full view of the system.
+        inspector_iniset $1 username admin
+        inspector_iniset $1 password $ADMIN_PASSWORD
+        inspector_iniset $1 system_scope all
+    else
+        inspector_iniset $1 username $IRONIC_INSPECTOR_ADMIN_USER
+        inspector_iniset $1 password $SERVICE_PASSWORD
+        inspector_iniset $1 project_name $SERVICE_PROJECT_NAME
+        inspector_iniset $1 project_domain_id default
+    fi
     inspector_iniset $1 user_domain_id default
-    inspector_iniset $1 project_domain_id default
     inspector_iniset $1 cafile $SSL_BUNDLE_FILE
     inspector_iniset $1 region_name $REGION_NAME
 }
@@ -335,7 +356,7 @@ function configure_inspector {
     if [[ "$IRONIC_INSPECTOR_STANDALONE" == "False" ]]; then
       # memcached listens localhost instead of $SERVICE_HOST, which is exactly the default value,
       # but set explicitly in case that changed.
-      inspector_iniset coordination backend_url "memcached://localhost:11211"
+      inspector_iniset coordination backend_url "memcached://127.0.0.1:11211"
     fi
 
     if is_service_enabled swift; then
@@ -396,6 +417,12 @@ function configure_inspector {
         configure_inspector_pxe_filter_dnsmasq
     fi
 
+    # Set if inspector should also be running in a scope enforced mode.
+    if [[ "$IRONIC_INSPECTOR_ENFORCE_SCOPE" == "True" ]]; then
+        inspector_iniset oslo_policy enforce_scope true
+        inspector_iniset oslo_policy enforce_new_defaults true
+    fi
+
 }
 
 function configure_inspector_swift {
@@ -413,7 +440,11 @@ interface=$IRONIC_INSPECTOR_INTERFACE
 bind-interfaces
 dhcp-range=$IRONIC_INSPECTOR_DHCP_RANGE
 dhcp-match=ipxe,175
-dhcp-boot=tag:!ipxe,undionly.kpxe
+dhcp-match=set:efi,option:client-arch,7
+dhcp-match=set:efi,option:client-arch,9
+dhcp-match=set:efi,option:client-arch,11
+dhcp-boot=tag:efi,tag:!ipxe,snponly.efi
+dhcp-boot=tag:!efi,tag:!ipxe,undionly.kpxe
 dhcp-boot=tag:ipxe,http://$IRONIC_HTTP_SERVER:$IRONIC_HTTP_PORT/ironic-inspector.ipxe
 dhcp-sequential-ip
 EOF
@@ -424,6 +455,10 @@ port=0
 interface=$IRONIC_INSPECTOR_INTERFACE
 bind-interfaces
 dhcp-range=$IRONIC_INSPECTOR_DHCP_RANGE
+dhcp-match=set:efi,option:client-arch,7
+dhcp-match=set:efi,option:client-arch,9
+dhcp-match=set:efi,option:client-arch,11
+dhcp-boot=tag:efi,bootx64.efi
 dhcp-boot=pxelinux.0
 dhcp-sequential-ip
 EOF
