@@ -28,7 +28,7 @@ from sqlalchemy import (Boolean, Column, DateTime, Enum, ForeignKey,
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy import orm
 
-from ironic_inspector import conf  # noqa
+
 from ironic_inspector import introspection_state as istate
 
 
@@ -91,6 +91,7 @@ class Rule(Base):
     description = Column(Text)
     # NOTE(dtantsur): in the future we might need to temporary disable a rule
     disabled = Column(Boolean, default=False)
+    scope = Column(String(255), nullable=True)
 
     conditions = orm.relationship('RuleCondition', lazy='joined',
                                   order_by='RuleCondition.id',
@@ -173,7 +174,9 @@ def _create_context_manager():
     _ctx_mgr = enginefacade.transaction_context()
     # TODO(aarefiev): enable foreign keys for SQLite once all unit
     #                 tests with failed constraint will be fixed.
-    _ctx_mgr.configure(sqlite_fk=False)
+    # FIXME(dtantsur): we need to remove reliance on autocommit semantics ASAP
+    # since it's not compatible with SQLAlchemy 2.0
+    _ctx_mgr.configure(sqlite_fk=False, __autocommit=True)
 
     return _ctx_mgr
 
