@@ -39,12 +39,13 @@ CONF = ironic_inspector.conf.CONF
 
 
 _app = flask.Flask(__name__)
+_app.url_map.strict_slashes = False
 _wsgi_app = _app.wsgi_app
 
 LOG = utils.getProcessingLogger(__name__)
 
 MINIMUM_API_VERSION = (1, 0)
-CURRENT_API_VERSION = (1, 17)
+CURRENT_API_VERSION = (1, 18)
 DEFAULT_API_VERSION = CURRENT_API_VERSION
 _LOGGING_EXCLUDED_KEYS = ('logs',)
 
@@ -285,11 +286,6 @@ def api(path, is_public_api=False, rule=None, verb_to_rule_map=None,
                              and strings to format the 'rule' string with
     :param kwargs: all the rest kwargs are passed to flask app.route
     """
-    # Force uniform behavior with regards to trailing slashes
-    if not path.endswith('/'):
-        path = path + '/'
-    flask_kwargs['strict_slashes'] = False
-
     def outer(func):
         @_app.route(path, **flask_kwargs)
         @convert_exceptions
@@ -329,7 +325,6 @@ def api_root():
      methods=['GET'])
 def version_root(version):
     pat = re.compile(r'^\/%s\/[^\/]*?/?$' % version)
-
     resources = []
     for url in _app.url_map.iter_rules():
         if pat.match(str(url)):
@@ -394,7 +389,8 @@ def api_introspection(node_id):
 def api_introspection_statuses():
     nodes = node_cache.get_node_list(
         marker=api_tools.marker_field(),
-        limit=api_tools.limit_field(default=CONF.api_max_limit)
+        limit=api_tools.limit_field(default=CONF.api_max_limit),
+        state=api_tools.state_field()
     )
     data = {
         'introspection': [generate_introspection_status(node)
