@@ -19,7 +19,6 @@ import jsonschema
 from oslo_db import exception as db_exc
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
-import six
 from sqlalchemy import orm
 
 from ironic_inspector.common.i18n import _
@@ -102,17 +101,19 @@ def actions_schema():
 class IntrospectionRule(object):
     """High-level class representing an introspection rule."""
 
-    def __init__(self, uuid, conditions, actions, description):
+    def __init__(self, uuid, conditions, actions, description, scope=None):
         """Create rule object from database data."""
         self._uuid = uuid
         self._conditions = conditions
         self._actions = actions
         self._description = description
+        self._scope = scope
 
     def as_dict(self, short=False):
         result = {
             'uuid': self._uuid,
             'description': self._description,
+            'scope': self._scope
         }
 
         if not short:
@@ -212,6 +213,26 @@ class IntrospectionRule(object):
         LOG.debug('Successfully applied actions',
                   node_info=node_info, data=data)
 
+    def check_scope(self, node_info):
+        """Check if node's scope falls under rule._scope and rule is applicable
+
+        :param node_info: a NodeInfo object
+        :returns: True if conditions match, otherwise False
+        """
+        if not self._scope:
+            LOG.debug('Rule "%s" is global and will be applied if conditions '
+                      'are met.', self._description)
+            return True
+        if self._scope == node_info.node().properties.get('inspection_scope'):
+            LOG.debug('Rule "%s" and node have a matching scope "%s and will '
+                      'be applied if conditions are met.', self._description,
+                      self._scope)
+            return True
+        else:
+            LOG.debug('Rule\'s "%s" scope "%s" does not match node\'s scope. '
+                      'Rule will be ignored', self._description, self._scope)
+            return False
+
 
 def _format_value(value, data):
     """Apply parameter formatting to a value.
@@ -225,7 +246,7 @@ def _format_value(value, data):
     :param data: Introspection data.
     :returns: `value`, formatted with the parameters from `data`.
     """
-    if isinstance(value, six.string_types):
+    if isinstance(value, str):
         # NOTE(aarefiev): verify provided value with introspection
         # data format specifications.
         # TODO(aarefiev): simple verify on import rule time.
@@ -237,7 +258,7 @@ def _format_value(value, data):
                               {'val': value, 'e': e}, data=data)
     elif isinstance(value, dict):
         return {_format_value(k, data): _format_value(v, data)
-                for k, v in six.iteritems(value)}
+                for k, v in value.items()}
     elif isinstance(value, list):
         return [_format_value(v, data) for v in value]
     else:
@@ -339,7 +360,7 @@ def _validate_actions(actions_json):
 
 
 def create(conditions_json, actions_json, uuid=None,
-           description=None):
+           description=None, scope=None):
     """Create a new rule in database.
 
     :param conditions_json: list of dicts with the following keys:
@@ -351,13 +372,16 @@ def create(conditions_json, actions_json, uuid=None,
                          Other keys are stored as is.
     :param uuid: rule UUID, will be generated if empty
     :param description: human-readable rule description
+    :param scope: if scope on node and rule matches, rule applies;
+                  if its empty, rule applies to all nodes.
     :returns: new IntrospectionRule object
     :raises: utils.Error on failure
     """
     uuid = uuid or uuidutils.generate_uuid()
     LOG.debug('Creating rule %(uuid)s with description "%(descr)s", '
-              'conditions %(conditions)s and actions %(actions)s',
-              {'uuid': uuid, 'descr': description,
+              'conditions %(conditions)s, scope "%(scope)s"'
+              ' and actions %(actions)s',
+              {'uuid': uuid, 'descr': description, 'scope': scope,
                'conditions': conditions_json, 'actions': actions_json})
 
     conditions = _validate_conditions(conditions_json)
@@ -365,8 +389,8 @@ def create(conditions_json, actions_json, uuid=None,
 
     try:
         with db.ensure_transaction() as session:
-            rule = db.Rule(uuid=uuid, description=description,
-                           disabled=False, created_at=timeutils.utcnow())
+            rule = db.Rule(uuid=uuid, description=description, disabled=False,
+                           created_at=timeutils.utcnow(), scope=scope)
 
             for field, op, multiple, invert, params in conditions:
                 rule.conditions.append(db.RuleCondition(op=op,
@@ -386,12 +410,14 @@ def create(conditions_json, actions_json, uuid=None,
         raise utils.Error(_('Rule with UUID %s already exists') % uuid,
                           code=409)
 
-    LOG.info('Created rule %(uuid)s with description "%(descr)s"',
-             {'uuid': uuid, 'descr': description})
+    LOG.info('Created rule %(uuid)s with description "%(descr)s" '
+             'and scope "%(scope)s"',
+             {'uuid': uuid, 'descr': description, 'scope': scope})
     return IntrospectionRule(uuid=uuid,
                              conditions=rule.conditions,
                              actions=rule.actions,
-                             description=description)
+                             description=description,
+                             scope=rule.scope)
 
 
 def get(uuid):
@@ -403,7 +429,8 @@ def get(uuid):
 
     return IntrospectionRule(uuid=rule.uuid, actions=rule.actions,
                              conditions=rule.conditions,
-                             description=rule.description)
+                             description=rule.description,
+                             scope=rule.scope)
 
 
 def get_all():
@@ -411,7 +438,8 @@ def get_all():
     query = db.model_query(db.Rule).order_by(db.Rule.created_at)
     return [IntrospectionRule(uuid=rule.uuid, actions=rule.actions,
                               conditions=rule.conditions,
-                              description=rule.description)
+                              description=rule.description,
+                              scope=rule.scope)
             for rule in query]
 
 
@@ -453,7 +481,8 @@ def apply(node_info, data):
 
     to_apply = []
     for rule in rules:
-        if rule.check_conditions(node_info, data):
+        if (rule.check_scope(node_info) and
+                rule.check_conditions(node_info, data)):
             to_apply.append(rule)
 
     if to_apply:
