@@ -11,33 +11,32 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import eventlet
-
-import contextlib  # noqa
+import contextlib
 import copy
 import datetime
+import functools
 import json
 import os
+import sys
 import tempfile
 import time
 import unittest
+from unittest import mock
+import urllib
 
+import eventlet
 import fixtures
-import mock
 from oslo_config import cfg
 from oslo_config import fixture as config_fixture
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
 import pytz
 import requests
-import six
-from six.moves import urllib
 
 from ironic_inspector.cmd import all as inspector_cmd
+from ironic_inspector.cmd import dbsync
 from ironic_inspector.common import ironic as ir_utils
-from ironic_inspector.common import swift
 from ironic_inspector import db
-from ironic_inspector import dbsync
 from ironic_inspector import introspection_state as istate
 from ironic_inspector import main
 from ironic_inspector import node_cache
@@ -57,10 +56,12 @@ driver = noop
 debug = True
 introspection_delay = 0
 auth_strategy=noauth
+transport_url=fake://
 [database]
 connection = sqlite:///%(db_file)s
 [processing]
 processing_hooks=$default_processing_hooks,lldp_basic
+store_data = database
 """
 
 
@@ -86,7 +87,7 @@ def get_error(response):
 
 def _query_string(*field_names):
     def outer(func):
-        @six.wraps(func)
+        @functools.wraps(func)
         def inner(*args, **kwargs):
             queries = []
             for field_name in field_names:
@@ -113,9 +114,9 @@ class Base(base.NodeTest):
         self.cli_fixture = self.useFixture(
             fixtures.MockPatchObject(ir_utils, 'get_client'))
         self.cli = self.cli_fixture.mock.return_value
-        self.cli.node.get.return_value = self.node
-        self.cli.node.update.return_value = self.node
-        self.cli.node.list.return_value = [self.node]
+        self.cli.get_node.return_value = self.node
+        self.cli.patch_node.return_value = self.node
+        self.cli.nodes.return_value = [self.node]
 
         self.patch = [
             {'op': 'add', 'path': '/properties/cpus', 'value': '4'},
@@ -176,6 +177,11 @@ class Base(base.NodeTest):
     def call_get_status(self, uuid, **kwargs):
         return self.call('get', '/v1/introspection/%s' % uuid, **kwargs).json()
 
+    def call_get_data(self, uuid, processed=True, **kwargs):
+        return self.call('get', '/v1/introspection/%s/data%s'
+                         % (uuid, '' if processed else '/unprocessed'),
+                         **kwargs).json()
+
     @_query_string('marker', 'limit')
     def call_get_statuses(self, query_string='', **kwargs):
         path = '/v1/introspection'
@@ -244,8 +250,8 @@ class Test(Base):
     def test_bmc(self):
         self.call_introspect(self.uuid)
         eventlet.greenthread.sleep(DEFAULT_SLEEP)
-        self.cli.node.set_power_state.assert_called_once_with(self.uuid,
-                                                              'reboot')
+        self.cli.set_node_power_state.assert_called_once_with(self.uuid,
+                                                              'rebooting')
 
         status = self.call_get_status(self.uuid)
         self.check_status(status, finished=False, state=istate.States.waiting)
@@ -254,12 +260,12 @@ class Test(Base):
         self.assertEqual({'uuid': self.uuid}, res)
         eventlet.greenthread.sleep(DEFAULT_SLEEP)
 
-        self.cli.node.update.assert_called_with(self.uuid, mock.ANY)
-        self.assertCalledWithPatch(self.patch, self.cli.node.update)
-        self.cli.port.create.assert_called_once_with(
+        self.cli.patch_node.assert_called_with(self.uuid, mock.ANY)
+        self.assertCalledWithPatch(self.patch, self.cli.patch_node)
+        self.cli.create_port.assert_called_once_with(
             node_uuid=self.uuid, address='11:22:33:44:55:66', extra={},
-            pxe_enabled=True)
-        self.assertTrue(self.cli.node.set_boot_device.called)
+            is_pxe_enabled=True)
+        self.assertTrue(self.cli.set_node_boot_device.called)
 
         status = self.call_get_status(self.uuid)
         self.check_status(status, finished=True, state=istate.States.finished)
@@ -270,22 +276,22 @@ class Test(Base):
 
         uuid_to_delete = uuidutils.generate_uuid()
         uuid_to_update = uuidutils.generate_uuid()
-        # Two ports already exist: one with incorrect pxe_enabled, the other
+        # Two ports already exist: one with incorrect is_pxe_enabled, the other
         # should be deleted.
-        self.cli.node.list_ports.return_value = [
-            mock.Mock(address=self.macs[1], uuid=uuid_to_update,
-                      node_uuid=self.uuid, extra={}, pxe_enabled=True),
-            mock.Mock(address='foobar', uuid=uuid_to_delete,
-                      node_uuid=self.uuid, extra={}, pxe_enabled=True),
+        self.cli.ports.return_value = [
+            mock.Mock(address=self.macs[1], id=uuid_to_update,
+                      node_id=self.uuid, extra={}, is_pxe_enabled=True),
+            mock.Mock(address='foobar', id=uuid_to_delete,
+                      node_id=self.uuid, extra={}, is_pxe_enabled=True),
         ]
         # Two more ports are created, one with client_id. Make sure the
         # returned object has the same properties as requested in create().
-        self.cli.port.create.side_effect = mock.Mock
+        self.cli.create_port.side_effect = mock.Mock
 
         self.call_introspect(self.uuid)
         eventlet.greenthread.sleep(DEFAULT_SLEEP)
-        self.cli.node.set_power_state.assert_called_once_with(self.uuid,
-                                                              'reboot')
+        self.cli.set_node_power_state.assert_called_once_with(self.uuid,
+                                                              'rebooting')
 
         status = self.call_get_status(self.uuid)
         self.check_status(status, finished=False, state=istate.States.waiting)
@@ -294,19 +300,57 @@ class Test(Base):
         self.assertEqual({'uuid': self.uuid}, res)
         eventlet.greenthread.sleep(DEFAULT_SLEEP)
 
-        self.cli.node.update.assert_called_with(self.uuid, mock.ANY)
-        self.assertCalledWithPatch(self.patch, self.cli.node.update)
+        self.cli.patch_node.assert_called_with(self.uuid, mock.ANY)
+        self.assertCalledWithPatch(self.patch, self.cli.patch_node)
         calls = [
             mock.call(node_uuid=self.uuid, address=self.macs[0],
-                      extra={}, pxe_enabled=True),
+                      extra={}, is_pxe_enabled=True),
             mock.call(node_uuid=self.uuid, address=self.macs[2],
-                      extra={'client-id': self.client_id}, pxe_enabled=False),
+                      extra={'client-id': self.client_id},
+                      is_pxe_enabled=False),
         ]
-        self.cli.port.create.assert_has_calls(calls, any_order=True)
-        self.cli.port.delete.assert_called_once_with(uuid_to_delete)
-        self.cli.port.update.assert_called_once_with(
-            uuid_to_update,
-            [{'op': 'replace', 'path': '/pxe_enabled', 'value': False}])
+        self.cli.create_port.assert_has_calls(calls, any_order=True)
+        self.cli.delete_port.assert_called_once_with(uuid_to_delete)
+
+        status = self.call_get_status(self.uuid)
+        self.check_status(status, finished=True, state=istate.States.finished)
+
+    def test_port_not_update_pxe_enabled(self):
+        cfg.CONF.set_override('add_ports', 'active', 'processing')
+        cfg.CONF.set_override('keep_ports', 'added', 'processing')
+        cfg.CONF.set_override('update_pxe_enabled', False, 'processing')
+
+        uuid_to_update = uuidutils.generate_uuid()
+        # One port with incorrect pxe_enabled.
+        self.cli.ports.return_value = [
+            mock.Mock(address=self.macs[0], id=uuid_to_update,
+                      node_id=self.uuid, extra={}, is_pxe_enabled=False)
+        ]
+        # Two more ports are created, one with client_id. Make sure the
+        # returned object has the same properties as requested in create().
+        self.cli.create_port.side_effect = mock.Mock
+
+        self.call_introspect(self.uuid)
+        eventlet.greenthread.sleep(DEFAULT_SLEEP)
+        self.cli.set_node_power_state.assert_called_once_with(self.uuid,
+                                                              'rebooting')
+
+        status = self.call_get_status(self.uuid)
+        self.check_status(status, finished=False, state=istate.States.waiting)
+
+        res = self.call_continue(self.data)
+        self.assertEqual({'uuid': self.uuid}, res)
+        eventlet.greenthread.sleep(DEFAULT_SLEEP)
+
+        self.cli.patch_node.assert_called_with(self.uuid, mock.ANY)
+        self.assertCalledWithPatch(self.patch, self.cli.patch_node)
+        calls = [
+            mock.call(node_uuid=self.uuid, address=self.macs[2],
+                      extra={'client-id': self.client_id},
+                      is_pxe_enabled=True),
+        ]
+        self.assertFalse(self.cli.patch_port.called)
+        self.cli.create_port.assert_has_calls(calls, any_order=True)
 
         status = self.call_get_status(self.uuid)
         self.check_status(status, finished=True, state=istate.States.finished)
@@ -364,7 +408,7 @@ class Test(Base):
     def test_manage_boot(self):
         self.call_introspect(self.uuid, manage_boot=False)
         eventlet.greenthread.sleep(DEFAULT_SLEEP)
-        self.assertFalse(self.cli.node.set_power_state.called)
+        self.assertFalse(self.cli.set_node_power_state.called)
 
         status = self.call_get_status(self.uuid)
         self.check_status(status, finished=False, state=istate.States.waiting)
@@ -373,8 +417,8 @@ class Test(Base):
         self.assertEqual({'uuid': self.uuid}, res)
         eventlet.greenthread.sleep(DEFAULT_SLEEP)
 
-        self.cli.node.update.assert_called_with(self.uuid, mock.ANY)
-        self.assertFalse(self.cli.node.set_boot_device.called)
+        self.cli.patch_node.assert_called_with(self.uuid, mock.ANY)
+        self.assertFalse(self.cli.set_node_boot_device.called)
 
         status = self.call_get_status(self.uuid)
         self.check_status(status, finished=True, state=istate.States.finished)
@@ -388,7 +432,8 @@ class Test(Base):
                 {'op': 'eq', 'field': 'memory_mb', 'value': 1024},
             ],
             'actions': [{'action': 'fail', 'message': 'boom'}],
-            'description': 'Cool actions'
+            'description': 'Cool actions',
+            'scope': "sniper's scope"
         }
 
         res = self.call_add_rule(rule)
@@ -406,7 +451,8 @@ class Test(Base):
         res = self.call_list_rules()
         self.assertEqual(rule['links'], res[0].pop('links'))
         self.assertEqual([{'uuid': rule['uuid'],
-                           'description': 'Cool actions'}],
+                           'description': rule['description'],
+                           'scope': rule['scope']}],
                          res)
 
         res = self.call_get_rule(rule['uuid'])
@@ -461,7 +507,7 @@ class Test(Base):
                      'value': 'foo'},
                     {'action': 'fail', 'message': 'boom'}
                 ]
-            }
+            },
         ]
         for rule in rules:
             self.call_add_rule(rule)
@@ -471,7 +517,7 @@ class Test(Base):
         self.call_continue(self.data)
         eventlet.greenthread.sleep(DEFAULT_SLEEP)
 
-        self.cli.node.update.assert_any_call(
+        self.cli.patch_node.assert_any_call(
             self.uuid,
             [{'op': 'add', 'path': '/extra/foo', 'value': 'bar'}])
 
@@ -509,11 +555,11 @@ class Test(Base):
         self.call_continue(self.data)
         eventlet.greenthread.sleep(DEFAULT_SLEEP)
 
-        self.cli.node.update.assert_any_call(
+        self.cli.patch_node.assert_any_call(
             self.uuid,
             [{'op': 'add', 'path': '/extra/foo', 'value': 'bar'}])
 
-        self.cli.node.update.assert_any_call(
+        self.cli.patch_node.assert_any_call(
             self.uuid,
             [{'op': 'add', 'path': '/driver_info/ipmi_address',
               'value': self.data['inventory']['bmc_address']}])
@@ -523,8 +569,8 @@ class Test(Base):
 
         self.call_introspect(self.uuid)
         eventlet.greenthread.sleep(DEFAULT_SLEEP)
-        self.cli.node.set_power_state.assert_called_once_with(self.uuid,
-                                                              'reboot')
+        self.cli.set_node_power_state.assert_called_once_with(self.uuid,
+                                                              'rebooting')
 
         status = self.call_get_status(self.uuid)
         self.check_status(status, finished=False, state=istate.States.waiting)
@@ -533,10 +579,10 @@ class Test(Base):
         self.assertEqual({'uuid': self.uuid}, res)
         eventlet.greenthread.sleep(DEFAULT_SLEEP)
 
-        self.assertCalledWithPatch(self.patch_root_hints, self.cli.node.update)
-        self.cli.port.create.assert_called_once_with(
+        self.assertCalledWithPatch(self.patch_root_hints, self.cli.patch_node)
+        self.cli.create_port.assert_called_once_with(
             node_uuid=self.uuid, address='11:22:33:44:55:66', extra={},
-            pxe_enabled=True)
+            is_pxe_enabled=True)
 
         status = self.call_get_status(self.uuid)
         self.check_status(status, finished=True, state=istate.States.finished)
@@ -544,8 +590,8 @@ class Test(Base):
     def test_abort_introspection(self):
         self.call_introspect(self.uuid)
         eventlet.greenthread.sleep(DEFAULT_SLEEP)
-        self.cli.node.set_power_state.assert_called_once_with(self.uuid,
-                                                              'reboot')
+        self.cli.set_node_power_state.assert_called_once_with(self.uuid,
+                                                              'rebooting')
         status = self.call_get_status(self.uuid)
         self.check_status(status, finished=False, state=istate.States.waiting)
 
@@ -565,20 +611,11 @@ class Test(Base):
         # after releasing the node lock
         self.call('post', '/v1/continue', self.data, expect_error=400)
 
-    @mock.patch.object(swift, 'store_introspection_data', autospec=True)
-    @mock.patch.object(swift, 'get_introspection_data', autospec=True)
-    def test_stored_data_processing(self, get_mock, store_mock):
-        cfg.CONF.set_override('store_data', 'swift', 'processing')
-
-        # ramdisk data copy
-        # please mind the data is changed during processing
-        ramdisk_data = json.dumps(copy.deepcopy(self.data))
-        get_mock.return_value = ramdisk_data
-
+    def test_stored_data_processing(self):
         self.call_introspect(self.uuid)
         eventlet.greenthread.sleep(DEFAULT_SLEEP)
-        self.cli.node.set_power_state.assert_called_once_with(self.uuid,
-                                                              'reboot')
+        self.cli.set_node_power_state.assert_called_once_with(self.uuid,
+                                                              'rebooting')
 
         res = self.call_continue(self.data)
         self.assertEqual({'uuid': self.uuid}, res)
@@ -588,9 +625,16 @@ class Test(Base):
         inspect_started_at = timeutils.parse_isotime(status['started_at'])
         self.check_status(status, finished=True, state=istate.States.finished)
 
+        data = self.call_get_data(self.uuid)
+        self.assertEqual(self.data['inventory'], data['inventory'])
+        self.assertIn('all_interfaces', data)
+        raw = self.call_get_data(self.uuid, processed=False)
+        self.assertEqual(self.data['inventory'], raw['inventory'])
+        self.assertNotIn('all_interfaces', raw)
+
         res = self.call_reapply(self.uuid)
         self.assertEqual(202, res.status_code)
-        self.assertEqual('', res.text)
+        self.assertEqual('{}\n', res.text)
         eventlet.greenthread.sleep(DEFAULT_SLEEP)
 
         status = self.call_get_status(self.uuid)
@@ -600,50 +644,27 @@ class Test(Base):
         reapply_started_at = timeutils.parse_isotime(status['started_at'])
         self.assertLess(inspect_started_at, reapply_started_at)
 
-        # reapply request data
-        get_mock.assert_called_once_with(self.uuid,
-                                         suffix='UNPROCESSED')
-
-        # store ramdisk data, store processing result data, store
-        # reapply processing result data; the ordering isn't
-        # guaranteed as store ramdisk data runs in a background
-        # thread; hower, last call has to always be reapply processing
-        # result data
-        store_ramdisk_call = mock.call(mock.ANY, self.uuid,
-                                       suffix='UNPROCESSED')
-        store_processing_call = mock.call(mock.ANY, self.uuid,
-                                          suffix=None)
-        self.assertEqual(3, len(store_mock.call_args_list))
-        self.assertIn(store_ramdisk_call,
-                      store_mock.call_args_list[0:2])
-        self.assertIn(store_processing_call,
-                      store_mock.call_args_list[0:2])
-        self.assertEqual(store_processing_call,
-                         store_mock.call_args_list[2])
-
         # second reapply call
-        get_mock.return_value = ramdisk_data
         res = self.call_reapply(self.uuid)
         self.assertEqual(202, res.status_code)
-        self.assertEqual('', res.text)
+        self.assertEqual('{}\n', res.text)
         eventlet.greenthread.sleep(DEFAULT_SLEEP)
 
-        # reapply saves the result
-        self.assertEqual(4, len(store_mock.call_args_list))
-        self.assertEqual(store_processing_call,
-                         store_mock.call_args_list[-1])
+        # Reapply with provided data
+        new_data = copy.deepcopy(self.data)
+        new_data['inventory']['cpu']['count'] = 42
+        res = self.call_reapply(self.uuid, data=new_data)
+        self.assertEqual(202, res.status_code)
+        self.assertEqual('{}\n', res.text)
+        eventlet.greenthread.sleep(DEFAULT_SLEEP)
 
-    @mock.patch.object(swift, 'store_introspection_data', autospec=True)
-    @mock.patch.object(swift, 'get_introspection_data', autospec=True)
-    def test_edge_state_transitions(self, get_mock, store_mock):
+        self.check_status(status, finished=True, state=istate.States.finished)
+
+        data = self.call_get_data(self.uuid)
+        self.assertEqual(42, data['inventory']['cpu']['count'])
+
+    def test_edge_state_transitions(self):
         """Assert state transitions work as expected in edge conditions."""
-        cfg.CONF.set_override('store_data', 'swift', 'processing')
-
-        # ramdisk data copy
-        # please mind the data is changed during processing
-        ramdisk_data = json.dumps(copy.deepcopy(self.data))
-        get_mock.return_value = ramdisk_data
-
         # multiple introspect calls
         self.call_introspect(self.uuid)
         self.call_introspect(self.uuid)
@@ -705,8 +726,8 @@ class Test(Base):
 
         self.call_introspect(self.uuid)
         eventlet.greenthread.sleep(DEFAULT_SLEEP)
-        self.cli.node.set_power_state.assert_called_once_with(self.uuid,
-                                                              'reboot')
+        self.cli.set_node_power_state.assert_called_once_with(self.uuid,
+                                                              'rebooting')
 
         status = self.call_get_status(self.uuid)
         self.check_status(status, finished=False, state=istate.States.waiting)
@@ -715,27 +736,20 @@ class Test(Base):
         self.assertEqual({'uuid': self.uuid}, res)
         eventlet.greenthread.sleep(DEFAULT_SLEEP)
 
-        self.cli.node.update.assert_called_with(self.uuid, mock.ANY)
-        self.assertCalledWithPatch(self.patch, self.cli.node.update)
-        self.cli.port.create.assert_called_once_with(
+        self.cli.patch_node.assert_called_with(self.uuid, mock.ANY)
+        self.assertCalledWithPatch(self.patch, self.cli.patch_node)
+        self.cli.create_port.assert_called_once_with(
             node_uuid=self.uuid, extra={}, address='11:22:33:44:55:66',
-            pxe_enabled=True)
+            is_pxe_enabled=True)
 
         status = self.call_get_status(self.uuid)
         self.check_status(status, finished=True, state=istate.States.finished)
 
-    @mock.patch.object(swift, 'store_introspection_data', autospec=True)
-    @mock.patch.object(swift, 'get_introspection_data', autospec=True)
-    def test_lldp_plugin(self, get_mock, store_mock):
-        cfg.CONF.set_override('store_data', 'swift', 'processing')
-
-        ramdisk_data = json.dumps(copy.deepcopy(self.data))
-        get_mock.return_value = ramdisk_data
-
+    def test_lldp_plugin(self):
         self.call_introspect(self.uuid)
         eventlet.greenthread.sleep(DEFAULT_SLEEP)
-        self.cli.node.set_power_state.assert_called_once_with(self.uuid,
-                                                              'reboot')
+        self.cli.set_node_power_state.assert_called_once_with(self.uuid,
+                                                              'rebooting')
 
         status = self.call_get_status(self.uuid)
         self.check_status(status, finished=False, state=istate.States.waiting)
@@ -747,9 +761,7 @@ class Test(Base):
         status = self.call_get_status(self.uuid)
         self.check_status(status, finished=True, state=istate.States.finished)
 
-        # Verify that the lldp_processed data is written to swift
-        # as expected by the lldp plugin
-        updated_data = store_mock.call_args[0][0]
+        updated_data = self.call_get_data(self.uuid)
         lldp_out = updated_data['all_interfaces']['eth1']
 
         expected_chassis_id = "11:22:33:aa:bb:cc"
@@ -759,6 +771,46 @@ class Test(Base):
         self.assertEqual(expected_port_id,
                          lldp_out['lldp_processed']['switch_port_id'])
 
+    def test_update_unknown_active_node(self):
+        cfg.CONF.set_override('permit_active_introspection', True,
+                              'processing')
+        self.node.provision_state = 'active'
+        self.cli.ports.return_value = [
+            mock.Mock(address='11:22:33:44:55:66', node_id=self.node.uuid)
+        ]
+
+        # NOTE(dtantsur): we're not starting introspection in this test.
+        res = self.call_continue(self.data)
+        self.assertEqual({'uuid': self.uuid}, res)
+        eventlet.greenthread.sleep(DEFAULT_SLEEP)
+
+        self.cli.patch_node.assert_called_with(self.uuid, mock.ANY)
+        self.assertCalledWithPatch(self.patch, self.cli.patch_node)
+        self.assertFalse(self.cli.create_port.called)
+        self.assertFalse(self.cli.set_node_boot_device.called)
+
+        status = self.call_get_status(self.uuid)
+        self.check_status(status, finished=True, state=istate.States.finished)
+
+    def test_update_known_active_node(self):
+        # Start with a normal introspection as a pre-requisite
+        self.test_bmc()
+
+        self.cli.patch_node.reset_mock()
+        self.cli.set_node_boot_device.reset_mock()
+        self.cli.create_port.reset_mock()
+        # Provide some updates
+        self.data['inventory']['memory']['physical_mb'] = 16384
+        self.patch = [
+            {'op': 'add', 'path': '/properties/cpus', 'value': '4'},
+            {'path': '/properties/cpu_arch', 'value': 'x86_64', 'op': 'add'},
+            {'op': 'add', 'path': '/properties/memory_mb', 'value': '16384'},
+            {'path': '/properties/local_gb', 'value': '999', 'op': 'add'}
+        ]
+
+        # Then continue with active node test
+        self.test_update_unknown_active_node()
+
 
 @contextlib.contextmanager
 def mocked_server():
@@ -789,5 +841,10 @@ def mocked_server():
 
 
 if __name__ == '__main__':
+    if len(sys.argv) > 1:
+        test_name = sys.argv[1]
+    else:
+        test_name = None
+
     with mocked_server():
-        unittest.main(verbosity=2)
+        unittest.main(verbosity=2, defaultTest=test_name)
