@@ -13,10 +13,10 @@
 
 import collections
 import time
+from unittest import mock
 
 import fixtures
-from ironicclient import exceptions
-import mock
+from openstack import exceptions as os_exc
 from oslo_config import cfg
 
 from ironic_inspector.common import ironic as ir_utils
@@ -46,12 +46,24 @@ class BaseTest(test_base.NodeTest):
         driver_mock = driver_fixture.mock.return_value
         self.sync_filter_mock = driver_mock.sync
 
+        self.async_exc = None
+        executor_fixture = self.useFixture(fixtures.MockPatchObject(
+            utils, 'executor', autospec=True))
+        self.submit_mock = executor_fixture.mock.return_value.submit
+        self.submit_mock.side_effect = self._submit
+
     def _prepare(self, client_mock):
         cli = client_mock.return_value
-        cli.node.get.return_value = self.node
-        cli.node.validate.return_value = mock.Mock(power={'result': True})
+        cli.get_node.return_value = self.node
+        cli.validate_node.return_value = mock.Mock(power={'result': True})
         return cli
 
+    def _submit(self, func, *args, **kwargs):
+        if self.async_exc:
+            self.assertRaisesRegex(*self.async_exc, func, *args, **kwargs)
+        else:
+            return func(*args, **kwargs)
+
 
 @mock.patch.object(node_cache, 'start_introspection', autospec=True)
 @mock.patch.object(ir_utils, 'get_client', autospec=True)
@@ -62,22 +74,78 @@ class TestIntrospect(BaseTest):
 
         introspect.introspect(self.node.uuid)
 
-        cli.node.get.assert_called_once_with(self.uuid)
-        cli.node.validate.assert_called_once_with(self.uuid)
+        cli.get_node.assert_called_once_with(self.uuid)
+        cli.validate_node.assert_called_once_with(self.uuid, required='power')
 
         start_mock.assert_called_once_with(self.uuid,
-                                           bmc_address=self.bmc_address,
+                                           bmc_address=[self.bmc_address],
+                                           manage_boot=True,
+                                           ironic=cli)
+        self.node_info.ports.assert_called_once_with()
+        self.node_info.add_attribute.assert_called_once_with(
+            'mac', self.macs)
+        self.sync_filter_mock.assert_called_with(cli)
+        cli.set_node_boot_device.assert_called_once_with(
+            self.uuid, 'pxe', persistent=False)
+        cli.set_node_power_state.assert_called_once_with(self.uuid,
+                                                         'rebooting')
+        self.node_info.acquire_lock.assert_called_once_with()
+        self.node_info.release_lock.assert_called_once_with()
+
+    def test_ok_retries_node_locked(self, client_mock, start_mock):
+        cli = self._prepare(client_mock)
+        start_mock.return_value = self.node_info
+        cli.set_node_power_state.side_effect = [
+            os_exc.ConflictException("Locked"),
+            None]
+        introspect.introspect(self.node.uuid)
+
+        cli.get_node.assert_called_once_with(self.uuid)
+        cli.validate_node.assert_called_once_with(self.uuid, required='power')
+
+        start_mock.assert_called_once_with(self.uuid,
+                                           bmc_address=[self.bmc_address],
+                                           manage_boot=True,
+                                           ironic=cli)
+        self.node_info.ports.assert_called_once_with()
+        self.node_info.add_attribute.assert_called_once_with(
+            'mac', self.macs)
+        self.sync_filter_mock.assert_called_with(cli)
+        cli.set_node_boot_device.assert_called_once_with(
+            self.uuid, 'pxe', persistent=False)
+        cli.set_node_power_state.assert_has_calls([
+            mock.call(self.uuid, 'rebooting'),
+            mock.call(self.uuid, 'rebooting')])
+
+        self.node_info.acquire_lock.assert_called_once_with()
+        self.node_info.release_lock.assert_called_once_with()
+
+    @mock.patch.object(ir_utils, 'get_ipmi_address', autospec=True)
+    def test_resolved_bmc_address(self, ipmi_mock, client_mock, start_mock):
+        self.node.driver_info['ipmi_address'] = 'example.com'
+        addresses = ['93.184.216.34', '2606:2800:220:1:248:1893:25c8:1946']
+        ipmi_mock.return_value = ('example.com',) + tuple(addresses)
+        cli = self._prepare(client_mock)
+        start_mock.return_value = self.node_info
+
+        introspect.introspect(self.node.uuid)
+
+        cli.get_node.assert_called_once_with(self.uuid)
+        cli.validate_node.assert_called_once_with(self.uuid, required='power')
+
+        start_mock.assert_called_once_with(self.uuid,
+                                           bmc_address=addresses,
                                            manage_boot=True,
                                            ironic=cli)
         self.node_info.ports.assert_called_once_with()
         self.node_info.add_attribute.assert_called_once_with('mac',
                                                              self.macs)
         self.sync_filter_mock.assert_called_with(cli)
-        cli.node.set_boot_device.assert_called_once_with(self.uuid,
+        cli.set_node_boot_device.assert_called_once_with(self.uuid,
                                                          'pxe',
                                                          persistent=False)
-        cli.node.set_power_state.assert_called_once_with(self.uuid,
-                                                         'reboot')
+        cli.set_node_power_state.assert_called_once_with(self.uuid,
+                                                         'rebooting')
         self.node_info.acquire_lock.assert_called_once_with()
         self.node_info.release_lock.assert_called_once_with()
 
@@ -88,22 +156,22 @@ class TestIntrospect(BaseTest):
 
         introspect.introspect(self.node.uuid)
 
-        cli.node.get.assert_called_once_with(self.uuid)
-        cli.node.validate.assert_called_once_with(self.uuid)
+        cli.get_node.assert_called_once_with(self.uuid)
+        cli.validate_node.assert_called_once_with(self.uuid, required='power')
 
         start_mock.assert_called_once_with(self.uuid,
-                                           bmc_address=None,
+                                           bmc_address=[],
                                            manage_boot=True,
                                            ironic=cli)
         self.node_info.ports.assert_called_once_with()
         self.node_info.add_attribute.assert_called_once_with('mac',
                                                              self.macs)
         self.sync_filter_mock.assert_called_with(cli)
-        cli.node.set_boot_device.assert_called_once_with(self.uuid,
+        cli.set_node_boot_device.assert_called_once_with(self.uuid,
                                                          'pxe',
                                                          persistent=False)
-        cli.node.set_power_state.assert_called_once_with(self.uuid,
-                                                         'reboot')
+        cli.set_node_power_state.assert_called_once_with(self.uuid,
+                                                         'rebooting')
         self.node_info.acquire_lock.assert_called_once_with()
         self.node_info.release_lock.assert_called_once_with()
 
@@ -116,28 +184,30 @@ class TestIntrospect(BaseTest):
             introspect.introspect(self.node.uuid)
 
         start_mock.assert_called_with(self.uuid,
-                                      bmc_address=self.bmc_address,
+                                      bmc_address=[self.bmc_address],
                                       manage_boot=True,
                                       ironic=cli)
 
     def test_power_failure(self, client_mock, start_mock):
         cli = self._prepare(client_mock)
-        cli.node.set_power_state.side_effect = exceptions.BadRequest()
+        cli.set_node_power_state.side_effect = os_exc.BadRequestException()
         start_mock.return_value = self.node_info
 
+        self.async_exc = (utils.Error, 'Failed to power on')
         introspect.introspect(self.node.uuid)
 
-        cli.node.get.assert_called_once_with(self.uuid)
+        cli.get_node.assert_called_once_with(self.uuid)
 
         start_mock.assert_called_once_with(self.uuid,
-                                           bmc_address=self.bmc_address,
+                                           bmc_address=[self.bmc_address],
                                            manage_boot=True,
                                            ironic=cli)
-        cli.node.set_boot_device.assert_called_once_with(self.uuid,
+        cli.set_node_boot_device.assert_called_once_with(self.uuid,
                                                          'pxe',
                                                          persistent=False)
-        cli.node.set_power_state.assert_called_once_with(self.uuid,
-                                                         'reboot')
+        power_call = mock.call(self.uuid, 'rebooting')
+        cli.set_node_power_state.assert_has_calls([
+            power_call, power_call, power_call, power_call, power_call])
         start_mock.return_value.finished.assert_called_once_with(
             introspect.istate.Events.error, error=mock.ANY)
         self.node_info.acquire_lock.assert_called_once_with()
@@ -148,15 +218,16 @@ class TestIntrospect(BaseTest):
         start_mock.return_value = self.node_info
         self.sync_filter_mock.side_effect = RuntimeError()
 
+        self.async_exc = (RuntimeError, '.*')
         introspect.introspect(self.node.uuid)
 
-        cli.node.get.assert_called_once_with(self.uuid)
+        cli.get_node.assert_called_once_with(self.uuid)
 
         start_mock.assert_called_once_with(self.uuid,
-                                           bmc_address=self.bmc_address,
+                                           bmc_address=[self.bmc_address],
                                            manage_boot=True,
                                            ironic=cli)
-        self.assertFalse(cli.node.set_boot_device.called)
+        self.assertFalse(cli.set_node_boot_device.called)
         start_mock.return_value.finished.assert_called_once_with(
             introspect.istate.Events.error, error=mock.ANY)
         self.node_info.acquire_lock.assert_called_once_with()
@@ -164,21 +235,22 @@ class TestIntrospect(BaseTest):
 
     def test_set_boot_device_failure(self, client_mock, start_mock):
         cli = self._prepare(client_mock)
-        cli.node.set_boot_device.side_effect = exceptions.BadRequest()
+        cli.set_node_boot_device.side_effect = os_exc.BadRequestException()
         start_mock.return_value = self.node_info
 
+        self.async_exc = (utils.Error, 'Failed to set boot device')
         introspect.introspect(self.node.uuid)
 
-        cli.node.get.assert_called_once_with(self.uuid)
+        cli.get_node.assert_called_once_with(self.uuid)
 
         start_mock.assert_called_once_with(self.uuid,
-                                           bmc_address=self.bmc_address,
+                                           bmc_address=[self.bmc_address],
                                            manage_boot=True,
                                            ironic=cli)
-        cli.node.set_boot_device.assert_called_once_with(self.uuid,
-                                                         'pxe',
-                                                         persistent=False)
-        cli.node.set_power_state.assert_not_called()
+        dev_call = mock.call(self.uuid, 'pxe', persistent=False)
+        cli.set_node_boot_device.assert_has_calls([
+            dev_call, dev_call, dev_call, dev_call, dev_call])
+        cli.set_node_power_state.assert_not_called()
         start_mock.return_value.finished.assert_called_once_with(
             introspect.istate.Events.error, error=mock.ANY)
         self.node_info.acquire_lock.assert_called_once_with()
@@ -194,16 +266,70 @@ class TestIntrospect(BaseTest):
         self.node_info.ports.assert_called_once_with()
 
         start_mock.assert_called_once_with(self.uuid,
-                                           bmc_address=self.bmc_address,
+                                           bmc_address=[self.bmc_address],
                                            manage_boot=True,
                                            ironic=cli)
         self.assertFalse(self.node_info.add_attribute.called)
         self.assertFalse(self.sync_filter_mock.called)
-        cli.node.set_boot_device.assert_called_once_with(self.uuid,
+        cli.set_node_boot_device.assert_called_once_with(self.uuid,
                                                          'pxe',
                                                          persistent=False)
-        cli.node.set_power_state.assert_called_once_with(self.uuid,
-                                                         'reboot')
+        cli.set_node_power_state.assert_called_once_with(self.uuid,
+                                                         'rebooting')
+
+    def test_forced_persistent_boot(self, client_mock, start_mock):
+        self.node.driver_info['force_persistent_boot_device'] = 'Always'
+        cli = self._prepare(client_mock)
+        start_mock.return_value = self.node_info
+
+        introspect.introspect(self.node.uuid)
+
+        cli.get_node.assert_called_once_with(self.uuid)
+        cli.validate_node.assert_called_once_with(self.uuid,
+                                                  required='power')
+
+        start_mock.assert_called_once_with(self.uuid,
+                                           bmc_address=[self.bmc_address],
+                                           manage_boot=True,
+                                           ironic=cli)
+        self.node_info.ports.assert_called_once_with()
+        self.node_info.add_attribute.assert_called_once_with('mac',
+                                                             self.macs)
+        self.sync_filter_mock.assert_called_with(cli)
+        cli.set_node_boot_device.assert_called_once_with(self.uuid,
+                                                         'pxe',
+                                                         persistent=True)
+        cli.set_node_power_state.assert_called_once_with(self.uuid,
+                                                         'rebooting')
+        self.node_info.acquire_lock.assert_called_once_with()
+        self.node_info.release_lock.assert_called_once_with()
+
+    def test_forced_persistent_boot_compat(self, client_mock, start_mock):
+        self.node.driver_info['force_persistent_boot_device'] = 'true'
+        cli = self._prepare(client_mock)
+        start_mock.return_value = self.node_info
+
+        introspect.introspect(self.node.uuid)
+
+        cli.get_node.assert_called_once_with(self.uuid)
+        cli.validate_node.assert_called_once_with(self.uuid,
+                                                  required='power')
+
+        start_mock.assert_called_once_with(self.uuid,
+                                           bmc_address=[self.bmc_address],
+                                           manage_boot=True,
+                                           ironic=cli)
+        self.node_info.ports.assert_called_once_with()
+        self.node_info.add_attribute.assert_called_once_with('mac',
+                                                             self.macs)
+        self.sync_filter_mock.assert_called_with(cli)
+        cli.set_node_boot_device.assert_called_once_with(self.uuid,
+                                                         'pxe',
+                                                         persistent=True)
+        cli.set_node_power_state.assert_called_once_with(self.uuid,
+                                                         'rebooting')
+        self.node_info.acquire_lock.assert_called_once_with()
+        self.node_info.release_lock.assert_called_once_with()
 
     def test_no_lookup_attrs(self, client_mock, start_mock):
         cli = self._prepare(client_mock)
@@ -211,13 +337,14 @@ class TestIntrospect(BaseTest):
         start_mock.return_value = self.node_info
         self.node_info.attributes = {}
 
+        self.async_exc = (utils.Error, 'No lookup attributes')
         introspect.introspect(self.uuid)
 
         self.node_info.ports.assert_called_once_with()
         self.node_info.finished.assert_called_once_with(
             introspect.istate.Events.error, error=mock.ANY)
         self.assertEqual(0, self.sync_filter_mock.call_count)
-        self.assertEqual(0, cli.node.set_power_state.call_count)
+        self.assertEqual(0, cli.set_node_power_state.call_count)
         self.node_info.acquire_lock.assert_called_once_with()
         self.node_info.release_lock.assert_called_once_with()
 
@@ -233,52 +360,50 @@ class TestIntrospect(BaseTest):
 
         self.node_info.ports.assert_called_once_with()
         self.assertFalse(self.node_info.finished.called)
-        cli.node.set_boot_device.assert_called_once_with(self.uuid,
+        cli.set_node_boot_device.assert_called_once_with(self.uuid,
                                                          'pxe',
                                                          persistent=False)
-        cli.node.set_power_state.assert_called_once_with(self.uuid,
-                                                         'reboot')
+        cli.set_node_power_state.assert_called_once_with(self.uuid,
+                                                         'rebooting')
 
     def test_failed_to_get_node(self, client_mock, start_mock):
         cli = client_mock.return_value
-        cli.node.get.side_effect = exceptions.NotFound()
+        cli.get_node.side_effect = os_exc.NotFoundException()
         self.assertRaisesRegex(utils.Error,
                                'Node %s was not found' % self.uuid,
                                introspect.introspect, self.uuid)
 
-        cli.node.get.side_effect = exceptions.BadRequest()
+        cli.get_node.side_effect = os_exc.BadRequestException()
         self.assertRaisesRegex(utils.Error,
-                               '%s: Bad Request' % self.uuid,
+                               'Cannot get node %s: Error' % self.uuid,
                                introspect.introspect, self.uuid)
 
         self.assertEqual(0, self.node_info.ports.call_count)
         self.assertEqual(0, self.sync_filter_mock.call_count)
-        self.assertEqual(0, cli.node.set_power_state.call_count)
+        self.assertEqual(0, cli.set_node_power_state.call_count)
         self.assertFalse(start_mock.called)
         self.assertFalse(self.node_info.acquire_lock.called)
 
     def test_failed_to_validate_node(self, client_mock, start_mock):
         cli = client_mock.return_value
-        cli.node.get.return_value = self.node
-        cli.node.validate.return_value = mock.Mock(power={'result': False,
-                                                          'reason': 'oops'})
-
+        cli.get_node.return_value = self.node
+        cli.validate_node.side_effect = os_exc.ValidationException()
         self.assertRaisesRegex(
             utils.Error,
-            'Failed validation of power interface',
+            'Failed validation of power interface: ValidationException',
             introspect.introspect, self.uuid)
 
-        cli.node.validate.assert_called_once_with(self.uuid)
+        cli.validate_node.assert_called_once_with(self.uuid, required='power')
         self.assertEqual(0, self.node_info.ports.call_count)
         self.assertEqual(0, self.sync_filter_mock.call_count)
-        self.assertEqual(0, cli.node.set_power_state.call_count)
+        self.assertEqual(0, cli.set_node_power_state.call_count)
         self.assertFalse(start_mock.called)
         self.assertFalse(self.node_info.acquire_lock.called)
 
     def test_wrong_provision_state(self, client_mock, start_mock):
         self.node.provision_state = 'active'
         cli = client_mock.return_value
-        cli.node.get.return_value = self.node
+        cli.get_node.return_value = self.node
 
         self.assertRaisesRegex(
             utils.Error, 'Invalid provision state for introspection: "active"',
@@ -286,21 +411,21 @@ class TestIntrospect(BaseTest):
 
         self.assertEqual(0, self.node_info.ports.call_count)
         self.assertEqual(0, self.sync_filter_mock.call_count)
-        self.assertEqual(0, cli.node.set_power_state.call_count)
+        self.assertEqual(0, cli.set_node_power_state.call_count)
         self.assertFalse(start_mock.called)
         self.assertFalse(self.node_info.acquire_lock.called)
 
     def test_inspect_wait_state_allowed(self, client_mock, start_mock):
         self.node.provision_state = 'inspect wait'
         cli = client_mock.return_value
-        cli.node.get.return_value = self.node
-        cli.node.validate.return_value = mock.Mock(power={'result': True})
+        cli.get_node.return_value = self.node
+        cli.validate_node.return_value = mock.Mock(power={'result': True})
 
         introspect.introspect(self.uuid)
 
         self.assertTrue(start_mock.called)
 
-    @mock.patch.object(time, 'time')
+    @mock.patch.object(time, 'time', autospec=True)
     def test_introspection_delay(self, time_mock, client_mock, start_mock):
         time_mock.return_value = 42
         introspect._LAST_INTROSPECTION_TIME = 40
@@ -312,15 +437,33 @@ class TestIntrospect(BaseTest):
         introspect.introspect(self.uuid)
 
         self.sleep_fixture.mock.assert_called_once_with(8)
-        cli.node.set_boot_device.assert_called_once_with(self.uuid,
+        cli.set_node_boot_device.assert_called_once_with(self.uuid,
                                                          'pxe',
                                                          persistent=False)
-        cli.node.set_power_state.assert_called_once_with(self.uuid,
-                                                         'reboot')
+        cli.set_node_power_state.assert_called_once_with(self.uuid,
+                                                         'rebooting')
         # updated to the current time.time()
         self.assertEqual(42, introspect._LAST_INTROSPECTION_TIME)
 
-    @mock.patch.object(time, 'time')
+    @mock.patch.object(time, 'time', autospec=True)
+    def test_introspection_no_delay_without_manage_boot(self, time_mock,
+                                                        client_mock,
+                                                        start_mock):
+        time_mock.return_value = 42
+        introspect._LAST_INTROSPECTION_TIME = 40
+        CONF.set_override('introspection_delay', 10)
+
+        self._prepare(client_mock)
+        start_mock.return_value = self.node_info
+        self.node_info.manage_boot = False
+
+        introspect.introspect(self.uuid, manage_boot=False)
+
+        self.assertFalse(self.sleep_fixture.mock.called)
+        # not updated
+        self.assertEqual(40, introspect._LAST_INTROSPECTION_TIME)
+
+    @mock.patch.object(time, 'time', autospec=True)
     def test_introspection_delay_not_needed(self, time_mock, client_mock,
                                             start_mock):
 
@@ -334,11 +477,11 @@ class TestIntrospect(BaseTest):
         introspect.introspect(self.uuid)
 
         self.sleep_fixture.mock().assert_not_called()
-        cli.node.set_boot_device.assert_called_once_with(self.uuid,
+        cli.set_node_boot_device.assert_called_once_with(self.uuid,
                                                          'pxe',
                                                          persistent=False)
-        cli.node.set_power_state.assert_called_once_with(self.uuid,
-                                                         'reboot')
+        cli.set_node_power_state.assert_called_once_with(self.uuid,
+                                                         'rebooting')
         # updated to the current time.time()
         self.assertEqual(100, introspect._LAST_INTROSPECTION_TIME)
 
@@ -349,19 +492,19 @@ class TestIntrospect(BaseTest):
 
         introspect.introspect(self.node.uuid, manage_boot=False)
 
-        cli.node.get.assert_called_once_with(self.uuid)
+        cli.get_node.assert_called_once_with(self.uuid)
 
         add_mock.assert_called_once_with(self.uuid,
-                                         bmc_address=self.bmc_address,
+                                         bmc_address=[self.bmc_address],
                                          manage_boot=False,
                                          ironic=cli)
         self.node_info.ports.assert_called_once_with()
         self.node_info.add_attribute.assert_called_once_with('mac',
                                                              self.macs)
         self.sync_filter_mock.assert_called_with(cli)
-        self.assertFalse(cli.node.validate.called)
-        self.assertFalse(cli.node.set_boot_device.called)
-        self.assertFalse(cli.node.set_power_state.called)
+        self.assertFalse(cli.validate_node.called)
+        self.assertFalse(cli.set_node_boot_device.called)
+        self.assertFalse(cli.set_node_power_state.called)
 
 
 @mock.patch.object(node_cache, 'get_node', autospec=True)
@@ -385,11 +528,11 @@ class TestAbort(BaseTest):
 
         introspect.abort(self.node.uuid)
 
-        get_mock.assert_called_once_with(self.uuid, ironic=cli,
-                                         locked=False)
+        get_mock.assert_called_once_with(self.uuid, ironic=cli)
         self.node_info.acquire_lock.assert_called_once_with(blocking=False)
         self.sync_filter_mock.assert_called_once_with(cli)
-        cli.node.set_power_state.assert_called_once_with(self.uuid, 'off')
+        cli.set_node_power_state.assert_called_once_with(self.uuid,
+                                                         'power off')
         self.node_info.finished.assert_called_once_with(
             introspect.istate.Events.abort_end, error='Canceled by operator')
         self.node_info.fsm_event.assert_has_calls(self.fsm_calls)
@@ -404,11 +547,10 @@ class TestAbort(BaseTest):
 
         introspect.abort(self.node.uuid)
 
-        get_mock.assert_called_once_with(self.uuid, ironic=cli,
-                                         locked=False)
+        get_mock.assert_called_once_with(self.uuid, ironic=cli)
         self.node_info.acquire_lock.assert_called_once_with(blocking=False)
         self.sync_filter_mock.assert_called_once_with(cli)
-        self.assertFalse(cli.node.set_power_state.called)
+        self.assertFalse(cli.set_node_power_state.called)
         self.node_info.finished.assert_called_once_with(
             introspect.istate.Events.abort_end, error='Canceled by operator')
         self.node_info.fsm_event.assert_has_calls(self.fsm_calls)
@@ -422,7 +564,7 @@ class TestAbort(BaseTest):
                                introspect.abort, self.uuid)
 
         self.assertEqual(0, self.sync_filter_mock.call_count)
-        self.assertEqual(0, cli.node.set_power_state.call_count)
+        self.assertEqual(0, cli.set_node_power_state.call_count)
         self.assertEqual(0, self.node_info.finished.call_count)
         self.assertEqual(0, self.node_info.fsm_event.call_count)
 
@@ -436,7 +578,7 @@ class TestAbort(BaseTest):
                                'retry later', introspect.abort, self.uuid)
 
         self.assertEqual(0, self.sync_filter_mock.call_count)
-        self.assertEqual(0, cli.node.set_power_state.call_count)
+        self.assertEqual(0, cli.set_node_power_state.call_count)
         self.assertEqual(0, self.node_info.finshed.call_count)
         self.assertEqual(0, self.node_info.fsm_event.call_count)
 
@@ -450,11 +592,11 @@ class TestAbort(BaseTest):
 
         introspect.abort(self.uuid)
 
-        get_mock.assert_called_once_with(self.uuid, ironic=cli,
-                                         locked=False)
+        get_mock.assert_called_once_with(self.uuid, ironic=cli)
         self.node_info.acquire_lock.assert_called_once_with(blocking=False)
         self.sync_filter_mock.assert_called_once_with(cli)
-        cli.node.set_power_state.assert_called_once_with(self.uuid, 'off')
+        cli.set_node_power_state.assert_called_once_with(self.uuid,
+                                                         'power off')
         self.node_info.finished.assert_called_once_with(
             introspect.istate.Events.abort_end, error='Canceled by operator')
         self.node_info.fsm_event.assert_has_calls(self.fsm_calls)
@@ -465,15 +607,15 @@ class TestAbort(BaseTest):
         self.node_info.acquire_lock.return_value = True
         self.node_info.started_at = time.time()
         self.node_info.finished_at = None
-        cli.node.set_power_state.side_effect = Exception('BadaBoom')
+        cli.set_node_power_state.side_effect = Exception('BadaBoom')
 
         introspect.abort(self.uuid)
 
-        get_mock.assert_called_once_with(self.uuid, ironic=cli,
-                                         locked=False)
+        get_mock.assert_called_once_with(self.uuid, ironic=cli)
         self.node_info.acquire_lock.assert_called_once_with(blocking=False)
         self.sync_filter_mock.assert_called_once_with(cli)
-        cli.node.set_power_state.assert_called_once_with(self.uuid, 'off')
+        cli.set_node_power_state.assert_called_once_with(self.uuid,
+                                                         'power off')
         self.node_info.finished.assert_called_once_with(
             introspect.istate.Events.abort_end, error='Canceled by operator')
         self.node_info.fsm_event.assert_has_calls(self.fsm_calls)
