@@ -21,7 +21,6 @@ from unittest import mock
 import automaton
 from openstack import exceptions as os_exc
 from oslo_config import cfg
-import oslo_db
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
 
@@ -976,12 +975,15 @@ class TestNodeCacheListNode(test_base.NodeTest):
     def setUp(self):
         super(TestNodeCacheListNode, self).setUp()
         self.uuid2 = uuidutils.generate_uuid()
+        self.uuid3 = uuidutils.generate_uuid()
         session = db.get_writer_session()
         with session.begin():
             db.Node(uuid=self.uuid,
                     started_at=datetime.datetime(1, 1, 2)).save(session)
             db.Node(uuid=self.uuid2, started_at=datetime.datetime(1, 1, 1),
                     finished_at=datetime.datetime(1, 1, 3)).save(session)
+            db.Node(uuid=self.uuid3, started_at=datetime.datetime(1, 1, 3),
+                    state='error').save(session)
 
     # mind please node(self.uuid).started_at > node(self.uuid2).started_at
     # and the result ordering is strict in node_cache.get_node_list newer first
@@ -989,12 +991,12 @@ class TestNodeCacheListNode(test_base.NodeTest):
     def test_list_node(self):
         nodes = node_cache.get_node_list()
 
-        self.assertEqual([self.uuid, self.uuid2],
+        self.assertEqual([self.uuid3, self.uuid, self.uuid2],
                          [node.uuid for node in nodes])
 
     def test_list_node_limit(self):
         nodes = node_cache.get_node_list(limit=1)
-        self.assertEqual([self.uuid], [node.uuid for node in nodes])
+        self.assertEqual([self.uuid3], [node.uuid for node in nodes])
 
     def test_list_node_marker(self):
         # get nodes started_at after node(self.uuid)
@@ -1005,6 +1007,15 @@ class TestNodeCacheListNode(test_base.NodeTest):
         self.assertRaises(utils.Error, node_cache.get_node_list,
                           marker='foo-bar')
 
+    def test_list_node_state(self):
+        nodes = node_cache.get_node_list(state=['error'])
+        self.assertEqual([self.uuid3], [node.uuid for node in nodes])
+
+    def test_list_node_state_multiple(self):
+        nodes = node_cache.get_node_list(state=['error', 'finished'])
+        self.assertEqual([self.uuid3, self.uuid, self.uuid2],
+                         [node.uuid for node in nodes])
+
 
 class TestNodeInfoVersionId(test_base.NodeStateTest):
     def test_get(self):
@@ -1051,11 +1062,6 @@ class TestNodeInfoState(test_base.NodeStateTest):
         row = db.model_query(db.Node).get(self.node_info.uuid)
         self.assertEqual(self.node_info.state, row.state)
 
-    def test_set_invalid_state(self):
-        self.assertRaisesRegex(oslo_db.exception.DBError,
-                               'constraint failed',
-                               self.node_info._set_state, 'foo')
-
     def test_commit(self):
         current_time = timeutils.utcnow()
         self.node_info.started_at = self.node_info.finished_at = current_time
