@@ -12,27 +12,29 @@
 # limitations under the License.
 
 import copy
+import functools
 import json
 import os
 import shutil
 import tempfile
+from unittest import mock
 
 import eventlet
 import fixtures
-from ironicclient import exceptions
-import mock
+from openstack import exceptions as os_exc
 from oslo_config import cfg
 from oslo_serialization import base64
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
-import six
 
 from ironic_inspector.common import ironic as ir_utils
+from ironic_inspector.common import swift
 from ironic_inspector import db
 from ironic_inspector import introspection_state as istate
 from ironic_inspector import node_cache
 from ironic_inspector.plugins import base as plugins_base
 from ironic_inspector.plugins import example as example_plugin
+from ironic_inspector.plugins import introspection_data as intros_data_plugin
 from ironic_inspector import process
 from ironic_inspector.pxe_filter import base as pxe_filter
 from ironic_inspector.test import base as test_base
@@ -71,14 +73,13 @@ class BaseProcessTest(BaseTest):
             started_at=self.started_at)
         self.node_info.finished = mock.Mock()
         self.find_mock.return_value = self.node_info
-        self.cli.node.get.return_value = self.node
+        self.cli.get_node.return_value = self.node
         self.process_mock = self.process_fixture.mock
         self.process_mock.return_value = self.fake_result_json
         self.addCleanup(self._cleanup_lock, self.node_info)
 
     def _cleanup_lock(self, node_info):
-        if node_info._locked:
-            node_info.release_lock()
+        node_info.release_lock()
 
 
 class TestProcess(BaseProcessTest):
@@ -87,33 +88,38 @@ class TestProcess(BaseProcessTest):
 
         self.assertEqual(self.fake_result_json, res)
 
-        self.find_mock.assert_called_once_with(bmc_address=self.bmc_address,
-                                               mac=mock.ANY)
+        self.find_mock.assert_called_once_with(
+            bmc_address=[self.bmc_address, self.bmc_v6address],
+            mac=mock.ANY)
         actual_macs = self.find_mock.call_args[1]['mac']
         self.assertEqual(sorted(self.all_macs), sorted(actual_macs))
-        self.cli.node.get.assert_called_once_with(self.uuid)
+        self.cli.get_node.assert_called_once_with(self.uuid)
         self.process_mock.assert_called_once_with(
             self.node_info, self.node, self.data)
 
     def test_no_ipmi(self):
         del self.inventory['bmc_address']
+        del self.inventory['bmc_v6address']
         process.process(self.data)
 
-        self.find_mock.assert_called_once_with(bmc_address=None, mac=mock.ANY)
+        self.find_mock.assert_called_once_with(bmc_address=[],
+                                               mac=mock.ANY)
         actual_macs = self.find_mock.call_args[1]['mac']
         self.assertEqual(sorted(self.all_macs), sorted(actual_macs))
-        self.cli.node.get.assert_called_once_with(self.uuid)
+        self.cli.get_node.assert_called_once_with(self.uuid)
         self.process_mock.assert_called_once_with(self.node_info, self.node,
                                                   self.data)
 
     def test_ipmi_not_detected(self):
         self.inventory['bmc_address'] = '0.0.0.0'
+        self.inventory['bmc_v6address'] = '::/0'
         process.process(self.data)
 
-        self.find_mock.assert_called_once_with(bmc_address=None, mac=mock.ANY)
+        self.find_mock.assert_called_once_with(bmc_address=[],
+                                               mac=mock.ANY)
         actual_macs = self.find_mock.call_args[1]['mac']
         self.assertEqual(sorted(self.all_macs), sorted(actual_macs))
-        self.cli.node.get.assert_called_once_with(self.uuid)
+        self.cli.get_node.assert_called_once_with(self.uuid)
         self.process_mock.assert_called_once_with(self.node_info, self.node,
                                                   self.data)
 
@@ -122,10 +128,12 @@ class TestProcess(BaseProcessTest):
         self.data['ipmi_address'] = '0.0.0.0'
         process.process(self.data)
 
-        self.find_mock.assert_called_once_with(bmc_address=None, mac=mock.ANY)
+        self.find_mock.assert_called_once_with(
+            bmc_address=[self.bmc_v6address],
+            mac=mock.ANY)
         actual_macs = self.find_mock.call_args[1]['mac']
         self.assertEqual(sorted(self.all_macs), sorted(actual_macs))
-        self.cli.node.get.assert_called_once_with(self.uuid)
+        self.cli.get_node.assert_called_once_with(self.uuid)
         self.process_mock.assert_called_once_with(self.node_info, self.node,
                                                   self.data)
 
@@ -134,16 +142,63 @@ class TestProcess(BaseProcessTest):
         self.assertRaisesRegex(utils.Error,
                                'not found',
                                process.process, self.data)
-        self.assertFalse(self.cli.node.get.called)
+        self.assertFalse(self.cli.get_node.called)
         self.assertFalse(self.process_mock.called)
 
+    @mock.patch.object(node_cache, 'record_node', autospec=True)
+    def test_not_found_in_cache_active_introspection(self, mock_record):
+        CONF.set_override('permit_active_introspection', True, 'processing')
+        self.find_mock.side_effect = utils.NotFoundInCacheError('not found')
+        self.cli.get_node.side_effect = os_exc.NotFoundException('boom')
+        self.cache_fixture.mock.acquire_lock = mock.Mock()
+        self.cache_fixture.mock.uuid = '1111'
+        self.cache_fixture.mock.finished_at = None
+        self.cache_fixture.mock.node = mock.Mock()
+        mock_record.return_value = self.cache_fixture.mock
+        res = process.process(self.data)
+
+        self.assertEqual(self.fake_result_json, res)
+        self.find_mock.assert_called_once_with(
+            bmc_address=[self.bmc_address, self.bmc_v6address],
+            mac=mock.ANY)
+        actual_macs = self.find_mock.call_args[1]['mac']
+        self.assertEqual(sorted(self.all_macs), sorted(actual_macs))
+        mock_record.assert_called_once_with(
+            bmc_addresses=['1.2.3.4',
+                           '2001:1234:1234:1234:1234:1234:1234:1234/64'],
+            macs=mock.ANY)
+        actual_macs = mock_record.call_args[1]['macs']
+        self.assertEqual(sorted(self.all_macs), sorted(actual_macs))
+        self.cli.get_node.assert_not_called()
+        self.process_mock.assert_called_once_with(
+            mock.ANY, mock.ANY, self.data)
+
+    def test_found_in_cache_active_introspection(self):
+        CONF.set_override('permit_active_introspection', True, 'processing')
+        self.node.provision_state = 'active'
+        self.cache_fixture.mock.acquire_lock = mock.Mock()
+        self.cache_fixture.mock.uuid = '1111'
+        self.cache_fixture.mock.finished_at = None
+        self.cache_fixture.mock.node = mock.Mock()
+        res = process.process(self.data)
+
+        self.assertEqual(self.fake_result_json, res)
+        self.find_mock.assert_called_once_with(
+            bmc_address=[self.bmc_address, self.bmc_v6address],
+            mac=mock.ANY)
+        actual_macs = self.find_mock.call_args[1]['mac']
+        self.assertEqual(sorted(self.all_macs), sorted(actual_macs))
+        self.cli.get_node.assert_called_once_with(self.uuid)
+        self.process_mock.assert_called_once_with(
+            mock.ANY, mock.ANY, self.data)
+
     def test_not_found_in_ironic(self):
-        self.cli.node.get.side_effect = exceptions.NotFound()
+        self.cli.get_node.side_effect = os_exc.NotFoundException()
 
         self.assertRaisesRegex(utils.Error,
                                'Node %s was not found' % self.uuid,
                                process.process, self.data)
-        self.cli.node.get.assert_called_once_with(self.uuid)
+        self.cli.get_node.assert_called_once_with(self.uuid)
         self.assertFalse(self.process_mock.called)
         self.node_info.finished.assert_called_once_with(
             istate.Events.error, error=mock.ANY)
@@ -179,7 +234,8 @@ class TestProcess(BaseProcessTest):
     def test_hook_unexpected_exceptions(self):
         for ext in plugins_base.processing_hooks_manager():
             patcher = mock.patch.object(ext.obj, 'before_processing',
-                                        side_effect=RuntimeError('boom'))
+                                        side_effect=RuntimeError('boom'),
+                                        autospec=True)
             patcher.start()
             self.addCleanup(lambda p=patcher: p.stop())
 
@@ -197,7 +253,8 @@ class TestProcess(BaseProcessTest):
         self.find_mock.side_effect = utils.Error('not found')
         for ext in plugins_base.processing_hooks_manager():
             patcher = mock.patch.object(ext.obj, 'before_processing',
-                                        side_effect=RuntimeError('boom'))
+                                        side_effect=RuntimeError('boom'),
+                                        autospec=True)
             patcher.start()
             self.addCleanup(lambda p=patcher: p.stop())
 
@@ -207,7 +264,6 @@ class TestProcess(BaseProcessTest):
         self.assertFalse(self.node_info.finished.called)
 
     def test_error_if_node_not_found_hook(self):
-        plugins_base._NOT_FOUND_HOOK_MGR = None
         self.find_mock.side_effect = utils.NotFoundInCacheError('BOOM')
         self.assertRaisesRegex(utils.Error,
                                'Look up error: BOOM',
@@ -219,7 +275,6 @@ class TestProcess(BaseProcessTest):
 class TestNodeNotFoundHook(BaseProcessTest):
     def test_node_not_found_hook_run_ok(self, hook_mock):
         CONF.set_override('node_not_found_hook', 'example', 'processing')
-        plugins_base._NOT_FOUND_HOOK_MGR = None
         self.find_mock.side_effect = utils.NotFoundInCacheError('BOOM')
         hook_mock.return_value = node_cache.NodeInfo(
             uuid=self.node.uuid,
@@ -230,7 +285,6 @@ class TestNodeNotFoundHook(BaseProcessTest):
 
     def test_node_not_found_hook_run_none(self, hook_mock):
         CONF.set_override('node_not_found_hook', 'example', 'processing')
-        plugins_base._NOT_FOUND_HOOK_MGR = None
         self.find_mock.side_effect = utils.NotFoundInCacheError('BOOM')
         hook_mock.return_value = None
         self.assertRaisesRegex(utils.Error,
@@ -240,7 +294,6 @@ class TestNodeNotFoundHook(BaseProcessTest):
 
     def test_node_not_found_hook_exception(self, hook_mock):
         CONF.set_override('node_not_found_hook', 'example', 'processing')
-        plugins_base._NOT_FOUND_HOOK_MGR = None
         self.find_mock.side_effect = utils.NotFoundInCacheError('BOOM')
         hook_mock.side_effect = Exception('Hook Error')
         self.assertRaisesRegex(utils.Error,
@@ -259,22 +312,13 @@ class TestUnprocessedData(BaseProcessTest):
 
         store_mock.assert_called_once_with(mock.ANY, expected)
 
-    @mock.patch.object(process.swift, 'SwiftAPI', autospec=True)
-    def test_save_unprocessed_data_failure(self, swift_mock):
+    def test_save_unprocessed_data_failure(self):
         CONF.set_override('store_data', 'swift', 'processing')
-        name = 'inspector_data-%s-%s' % (
-            self.uuid,
-            process._UNPROCESSED_DATA_STORE_SUFFIX
-        )
-
-        swift_conn = swift_mock.return_value
-        swift_conn.create_object.side_effect = utils.Error('Oops')
 
         res = process.process(self.data)
 
         # assert store failure doesn't break processing
         self.assertEqual(self.fake_result_json, res)
-        swift_conn.create_object.assert_called_once_with(name, mock.ANY)
 
 
 @mock.patch.object(example_plugin.ExampleProcessingHook, 'before_processing',
@@ -325,7 +369,7 @@ class TestStoreLogs(BaseProcessTest):
         self._check_contents()
 
     def test_store_find_node_error(self, hook_mock):
-        self.cli.node.get.side_effect = exceptions.NotFound('boom')
+        self.cli.get_node.side_effect = os_exc.NotFoundException('boom')
         self.assertRaises(utils.Error, process.process, self.data)
         self._check_contents()
 
@@ -389,11 +433,11 @@ class TestProcessNode(BaseTest):
         self.data['interfaces'] = self.valid_interfaces
         self.ports = self.all_ports
 
-        self.cli.node.get_boot_device.side_effect = (
+        self.cli.get_node.return_value.boot_device.side_effect = (
             [RuntimeError()] * self.validate_attempts + [None])
-        self.cli.port.create.side_effect = self.ports
-        self.cli.node.update.return_value = self.node
-        self.cli.node.list_ports.return_value = []
+        self.cli.create_port.side_effect = self.ports
+        self.cli.update_node.return_value = self.node
+        self.cli.ports.return_value = []
 
         self.useFixture(fixtures.MockPatchObject(
             pxe_filter, 'driver', autospec=True))
@@ -410,7 +454,8 @@ class TestProcessNode(BaseTest):
         ret_val = process._process_node(self.node_info, self.node, self.data)
         self.assertEqual(self.uuid, ret_val.get('uuid'))
 
-    @mock.patch.object(example_plugin.ExampleProcessingHook, 'before_update')
+    @mock.patch.object(example_plugin.ExampleProcessingHook, 'before_update',
+                       autospec=True)
     def test_wrong_provision_state(self, post_hook_mock):
         self.node.provision_state = 'active'
 
@@ -418,52 +463,81 @@ class TestProcessNode(BaseTest):
                           self.node_info, self.node, self.data)
         self.assertFalse(post_hook_mock.called)
 
-    @mock.patch.object(example_plugin.ExampleProcessingHook, 'before_update')
+    @mock.patch.object(example_plugin.ExampleProcessingHook, 'before_update',
+                       autospec=True)
     @mock.patch.object(node_cache.NodeInfo, 'finished', autospec=True)
     def test_ok(self, finished_mock, post_hook_mock):
         process._process_node(self.node_info, self.node, self.data)
 
-        self.cli.port.create.assert_any_call(node_uuid=self.uuid,
+        self.cli.create_port.assert_any_call(node_uuid=self.uuid,
                                              address=self.macs[0],
                                              extra={},
-                                             pxe_enabled=True)
-        self.cli.port.create.assert_any_call(node_uuid=self.uuid,
+                                             is_pxe_enabled=True)
+        self.cli.create_port.assert_any_call(node_uuid=self.uuid,
                                              address=self.macs[1],
                                              extra={},
-                                             pxe_enabled=False)
-        self.cli.node.set_power_state.assert_called_once_with(self.uuid, 'off')
-        self.assertFalse(self.cli.node.validate.called)
+                                             is_pxe_enabled=False)
+        self.cli.set_node_power_state.assert_called_once_with(self.uuid,
+                                                              'power off')
+        self.assertFalse(self.cli.validate_node.called)
 
-        post_hook_mock.assert_called_once_with(self.data, self.node_info)
+        post_hook_mock.assert_called_once_with(mock.ANY, self.data,
+                                               self.node_info)
+        finished_mock.assert_called_once_with(mock.ANY, istate.Events.finish)
+
+    @mock.patch.object(example_plugin.ExampleProcessingHook, 'before_update',
+                       autospec=True)
+    @mock.patch.object(node_cache.NodeInfo, 'finished', autospec=True)
+    def test_ok_node_active(self, finished_mock, post_hook_mock):
+        self.node.provision_state = 'active'
+        CONF.set_override('permit_active_introspection', True, 'processing')
+        process._process_node(self.node_info, self.node, self.data)
+
+        self.cli.create_port.assert_any_call(node_uuid=self.uuid,
+                                             address=self.macs[0],
+                                             extra={},
+                                             is_pxe_enabled=True)
+        self.cli.create_port.assert_any_call(node_uuid=self.uuid,
+                                             address=self.macs[1],
+                                             extra={},
+                                             is_pxe_enabled=False)
+
+        self.cli.set_node_power_state.assert_not_called()
+        self.assertFalse(self.cli.validate_node.called)
+
+        post_hook_mock.assert_called_once_with(mock.ANY, self.data,
+                                               self.node_info)
         finished_mock.assert_called_once_with(mock.ANY, istate.Events.finish)
 
     def test_port_failed(self):
-        self.cli.port.create.side_effect = (
-            [exceptions.Conflict()] + self.ports[1:])
+        self.cli.create_port.side_effect = (
+            [os_exc.ConflictException()] + self.ports[1:])
 
         process._process_node(self.node_info, self.node, self.data)
 
-        self.cli.port.create.assert_any_call(node_uuid=self.uuid,
+        self.cli.create_port.assert_any_call(node_uuid=self.uuid,
                                              address=self.macs[0],
-                                             extra={}, pxe_enabled=True)
-        self.cli.port.create.assert_any_call(node_uuid=self.uuid,
+                                             extra={}, is_pxe_enabled=True)
+        self.cli.create_port.assert_any_call(node_uuid=self.uuid,
                                              address=self.macs[1],
-                                             extra={}, pxe_enabled=False)
+                                             extra={}, is_pxe_enabled=False)
 
     @mock.patch.object(node_cache.NodeInfo, 'finished', autospec=True)
     def test_power_off_failed(self, finished_mock):
-        self.cli.node.set_power_state.side_effect = RuntimeError('boom')
+        self.cli.set_node_power_state.side_effect = RuntimeError('boom')
 
         process._process_node(self.node_info, self.node, self.data)
 
-        self.cli.node.set_power_state.assert_called_once_with(self.uuid, 'off')
+        self.cli.set_node_power_state.assert_called_once_with(self.uuid,
+                                                              'power off')
         finished_mock.assert_called_once_with(
             mock.ANY, istate.Events.error,
             error='Failed to power off node %s, check its power '
             'management configuration: boom' % self.uuid
         )
 
-    @mock.patch.object(example_plugin.ExampleProcessingHook, 'before_update')
+    @mock.patch.object(example_plugin.ExampleProcessingHook, 'before_update',
+                       autospec=True)
     @mock.patch.object(node_cache.NodeInfo, 'finished', autospec=True)
     def test_power_off_enroll_state(self, finished_mock, post_hook_mock):
         self.node.provision_state = 'enroll'
@@ -472,7 +546,7 @@ class TestProcessNode(BaseTest):
         process._process_node(self.node_info, self.node, self.data)
 
         self.assertTrue(post_hook_mock.called)
-        self.assertTrue(self.cli.node.set_power_state.called)
+        self.assertTrue(self.cli.set_node_power_state.called)
         finished_mock.assert_called_once_with(
             self.node_info, istate.Events.finish)
 
@@ -481,12 +555,21 @@ class TestProcessNode(BaseTest):
         CONF.set_override('power_off', False, 'processing')
         process._process_node(self.node_info, self.node, self.data)
 
+        self.assertFalse(self.cli.set_node_power_state.called)
+        finished_mock.assert_called_once_with(
+            self.node_info, istate.Events.finish)
+
+    @mock.patch.object(node_cache.NodeInfo, 'finished', autospec=True)
+    def test_no_manage_boot(self, finished_mock):
+        self.node_info._manage_boot = False
+        process._process_node(self.node_info, self.node, self.data)
+
         self.assertFalse(self.cli.node.set_power_state.called)
         finished_mock.assert_called_once_with(
             self.node_info, istate.Events.finish)
 
-    @mock.patch.object(process.swift, 'SwiftAPI', autospec=True)
-    def test_store_data(self, swift_mock):
+    @mock.patch.object(swift, 'SwiftAPI', autospec=True)
+    def test_store_data_with_swift(self, swift_mock):
         CONF.set_override('store_data', 'swift', 'processing')
         swift_conn = swift_mock.return_value
         name = 'inspector_data-%s' % self.uuid
@@ -498,8 +581,8 @@ class TestProcessNode(BaseTest):
         self.assertEqual(expected,
                          json.loads(swift_conn.create_object.call_args[0][1]))
 
-    @mock.patch.object(process.swift, 'SwiftAPI', autospec=True)
-    def test_store_data_no_logs(self, swift_mock):
+    @mock.patch.object(swift, 'SwiftAPI', autospec=True)
+    def test_store_data_no_logs_with_swift(self, swift_mock):
         CONF.set_override('store_data', 'swift', 'processing')
         swift_conn = swift_mock.return_value
         name = 'inspector_data-%s' % self.uuid
@@ -511,30 +594,34 @@ class TestProcessNode(BaseTest):
         self.assertNotIn('logs',
                          json.loads(swift_conn.create_object.call_args[0][1]))
 
-    @mock.patch.object(process.swift, 'SwiftAPI', autospec=True)
-    def test_store_data_location(self, swift_mock):
-        CONF.set_override('store_data', 'swift', 'processing')
-        CONF.set_override('store_data_location', 'inspector_data_object',
-                          'processing')
-        swift_conn = swift_mock.return_value
-        name = 'inspector_data-%s' % self.uuid
-        patch = [{'path': '/extra/inspector_data_object',
-                  'value': name, 'op': 'add'}]
-        expected = self.data
+    @mock.patch.object(node_cache, 'store_introspection_data', autospec=True)
+    def test_store_data_with_database(self, store_mock):
+        CONF.set_override('store_data', 'database', 'processing')
 
         process._process_node(self.node_info, self.node, self.data)
 
-        swift_conn.create_object.assert_called_once_with(name, mock.ANY)
-        self.assertEqual(expected,
-                         json.loads(swift_conn.create_object.call_args[0][1]))
-        self.cli.node.update.assert_any_call(self.uuid, patch)
+        data = intros_data_plugin._filter_data_excluded_keys(self.data)
+        store_mock.assert_called_once_with(self.node_info.uuid, data, True)
+        self.assertEqual(data, store_mock.call_args[0][1])
+
+    @mock.patch.object(node_cache, 'store_introspection_data', autospec=True)
+    def test_store_data_no_logs_with_database(self, store_mock):
+        CONF.set_override('store_data', 'database', 'processing')
+
+        self.data['logs'] = 'something'
+
+        process._process_node(self.node_info, self.node, self.data)
+
+        data = intros_data_plugin._filter_data_excluded_keys(self.data)
+        store_mock.assert_called_once_with(self.node_info.uuid, data, True)
+        self.assertNotIn('logs', store_mock.call_args[0][1])
 
 
 @mock.patch.object(process, '_reapply', autospec=True)
 @mock.patch.object(node_cache, 'get_node', autospec=True)
 class TestReapply(BaseTest):
     def prepare_mocks(func):
-        @six.wraps(func)
+        @functools.wraps(func)
         def wrapper(self, pop_mock, *args, **kw):
             pop_mock.return_value = node_cache.NodeInfo(
                 uuid=self.node.uuid,
@@ -553,12 +640,13 @@ class TestReapply(BaseTest):
     @prepare_mocks
     def test_ok(self, pop_mock, reapply_mock):
         process.reapply(self.uuid)
-        pop_mock.assert_called_once_with(self.uuid, locked=False)
+        pop_mock.assert_called_once_with(self.uuid)
         pop_mock.return_value.acquire_lock.assert_called_once_with(
             blocking=False
         )
 
-        reapply_mock.assert_called_once_with(pop_mock.return_value)
+        reapply_mock.assert_called_once_with(pop_mock.return_value,
+                                             introspection_data=None)
 
     @prepare_mocks
     def test_locking_failed(self, pop_mock, reapply_mock):
@@ -567,15 +655,26 @@ class TestReapply(BaseTest):
                                'Node locked, please, try again later',
                                process.reapply, self.uuid)
 
-        pop_mock.assert_called_once_with(self.uuid, locked=False)
+        pop_mock.assert_called_once_with(self.uuid)
         pop_mock.return_value.acquire_lock.assert_called_once_with(
             blocking=False
         )
 
+    @prepare_mocks
+    def test_reapply_with_data(self, pop_mock, reapply_mock):
+        process.reapply(self.uuid, data=self.data)
+        pop_mock.assert_called_once_with(self.uuid)
+        pop_mock.return_value.acquire_lock.assert_called_once_with(
+            blocking=False
+        )
+        reapply_mock.assert_called_once_with(pop_mock.return_value,
+                                             introspection_data=self.data)
 
-@mock.patch.object(example_plugin.ExampleProcessingHook, 'before_update')
+
+@mock.patch.object(example_plugin.ExampleProcessingHook, 'before_update',
+                   autospec=True)
 @mock.patch.object(process.rules, 'apply', autospec=True)
-@mock.patch.object(process.swift, 'SwiftAPI', autospec=True)
+@mock.patch.object(swift, 'SwiftAPI', autospec=True)
 @mock.patch.object(node_cache.NodeInfo, 'finished', autospec=True)
 @mock.patch.object(node_cache.NodeInfo, 'release_lock', autospec=True)
 class TestReapplyNode(BaseTest):
@@ -592,9 +691,9 @@ class TestReapplyNode(BaseTest):
                                              node=self.node)
         self.node_info.invalidate_cache = mock.Mock()
 
-        self.cli.port.create.side_effect = self.ports
-        self.cli.node.update.return_value = self.node
-        self.cli.node.list_ports.return_value = []
+        self.cli.create_port.side_effect = self.ports
+        self.cli.update_node.return_value = self.node
+        self.cli.ports.return_value = []
         self.node_info._state = istate.States.finished
         self.commit_fixture = self.useFixture(
             fixtures.MockPatchObject(node_cache.NodeInfo, 'commit',
@@ -605,12 +704,12 @@ class TestReapplyNode(BaseTest):
                 error=self.node_info.error).save(self.session)
 
     def call(self):
-        process._reapply(self.node_info)
+        process._reapply(self.node_info, introspection_data=self.data)
         # make sure node_info lock is released after a call
         self.node_info.release_lock.assert_called_once_with(self.node_info)
 
     def prepare_mocks(fn):
-        @six.wraps(fn)
+        @functools.wraps(fn)
         def wrapper(self, release_mock, finished_mock, swift_mock,
                     *args, **kw):
             finished_mock.side_effect = lambda *a, **kw: \
@@ -621,67 +720,46 @@ class TestReapplyNode(BaseTest):
 
     @prepare_mocks
     def test_ok(self, finished_mock, swift_mock, apply_mock, post_hook_mock):
-        swift_name = 'inspector_data-%s' % self.uuid
-        swift_mock.get_object.return_value = json.dumps(self.data)
-
         self.call()
 
         self.commit_fixture.mock.assert_called_once_with(self.node_info)
 
-        post_hook_mock.assert_called_once_with(mock.ANY, self.node_info)
-        swift_mock.create_object.assert_called_once_with(swift_name,
-                                                         mock.ANY)
-        swifted_data = json.loads(swift_mock.create_object.call_args[0][1])
+        post_hook_mock.assert_called_once_with(mock.ANY, mock.ANY,
+                                               self.node_info)
 
         self.node_info.invalidate_cache.assert_called_once_with()
-        apply_mock.assert_called_once_with(self.node_info, swifted_data)
+        apply_mock.assert_called_once_with(self.node_info, self.data)
 
         # assert no power operations were performed
-        self.assertFalse(self.cli.node.set_power_state.called)
+        self.assertFalse(self.cli.set_node_power_state.called)
         finished_mock.assert_called_once_with(
             self.node_info, istate.Events.finish)
 
         # asserting validate_interfaces was called
-        self.assertEqual(self.pxe_interfaces, swifted_data['interfaces'])
-        self.assertEqual([self.pxe_mac], swifted_data['macs'])
+        self.assertEqual(self.pxe_interfaces, self.data['interfaces'])
+        self.assertEqual([self.pxe_mac], self.data['macs'])
 
         # assert ports were created with whatever there was left
         # behind validate_interfaces
-        self.cli.port.create.assert_called_once_with(
+        self.cli.create_port.assert_called_once_with(
             node_uuid=self.uuid,
-            address=swifted_data['macs'][0],
+            address=self.data['macs'][0],
             extra={},
-            pxe_enabled=True
+            is_pxe_enabled=True
         )
 
-    @prepare_mocks
-    def test_get_incomming_data_exception(self, finished_mock, swift_mock,
-                                          apply_mock, post_hook_mock):
-        exc = Exception('Oops')
-        expected_error = ('Unexpected exception Exception while fetching '
-                          'unprocessed introspection data from Swift: Oops')
-        swift_mock.get_object.side_effect = exc
-        self.call()
-
-        self.commit_fixture.mock.assert_called_once_with(self.node_info)
-        self.assertFalse(swift_mock.create_object.called)
-        self.assertFalse(apply_mock.called)
-        self.assertFalse(post_hook_mock.called)
-        finished_mock.assert_called_once_with(
-            self.node_info, istate.Events.error, error=expected_error)
-
     @prepare_mocks
     def test_prehook_failure(self, finished_mock, swift_mock, apply_mock,
                              post_hook_mock):
         CONF.set_override('processing_hooks', 'example',
                           'processing')
-        plugins_base._HOOKS_MGR = None
 
         exc = Exception('Failed.')
         swift_mock.get_object.return_value = json.dumps(self.data)
 
         with mock.patch.object(example_plugin.ExampleProcessingHook,
-                               'before_processing') as before_processing_mock:
+                               'before_processing',
+                               autospec=True) as before_processing_mock:
             before_processing_mock.side_effect = exc
             self.call()
 
@@ -697,17 +775,3 @@ class TestReapplyNode(BaseTest):
         self.assertFalse(swift_mock.create_object.called)
         self.assertFalse(apply_mock.called)
         self.assertFalse(post_hook_mock.called)
-
-    @prepare_mocks
-    def test_generic_exception_creating_ports(self, finished_mock, swift_mock,
-                                              apply_mock, post_hook_mock):
-        swift_mock.get_object.return_value = json.dumps(self.data)
-        exc = Exception('Oops')
-        self.cli.port.create.side_effect = exc
-        self.call()
-
-        finished_mock.assert_called_once_with(
-            self.node_info, istate.Events.error, error=str(exc))
-        self.assertFalse(swift_mock.create_object.called)
-        self.assertFalse(apply_mock.called)
-        self.assertFalse(post_hook_mock.called)
